<?php

/**
 * JCH Optimize - Performs several front-end optimizations for fast downloads.
 *
 * @author    Samuel Marshall <samuel@jch-optimize.net>
 * @copyright Copyright (c) 2020 Samuel Marshall / JCH Optimize
 * @license   GNU/GPLv3, or later. See LICENSE file
 *
 * If LICENSE file missing, see <http://www.gnu.org/licenses/>.
 */

namespace JchOptimize\Controller;

use _JchOptimizeVendor\Joomla\Controller\AbstractController;
use JchOptimize\Core\Admin\Icons;
use JchOptimize\Core\PageCache\CaptureCache;
use JchOptimize\Joomla\Plugin\PluginHelper;
use JchOptimize\Model\Updates;
use JchOptimize\View\ControlPanelHtml;
use Joomla\CMS\Application\AdministratorApplication;
use Joomla\CMS\Language\Text;
use Joomla\CMS\Router\Route;
use Joomla\CMS\Uri\Uri;
use Joomla\DI\ContainerAwareInterface;
use Joomla\DI\ContainerAwareTrait;
use Joomla\Input\Input;

\defined('_JEXEC') or exit('Restricted Access');
class ControlPanel extends AbstractController implements ContainerAwareInterface
{
    use ContainerAwareTrait;

    private ControlPanelHtml $view;

    private Updates $updatesModel;

    private Icons $icons;

    /**
     * Constructor.
     */
    public function __construct(Updates $updatesModel, ControlPanelHtml $view, Icons $icons, Input $input = null, AdministratorApplication $app = null)
    {
        $this->updatesModel = $updatesModel;
        $this->view = $view;
        $this->icons = $icons;
        parent::__construct($input, $app);
    }

    public function execute()
    {
        $this->manageUpdates();
        if (\JCH_PRO) {
            // @see CaptureCache::updateHtaccess()
            $this->container->get(CaptureCache::class)->updateHtaccess();
        }
        $this->view->setData(['view' => 'ControlPanel', 'icons' => $this->icons]);
        $this->view->loadResources();
        $this->view->loadToolBar();
        if (!PluginHelper::isEnabled('system', 'jchoptimize')) {
            if (\JCH_PRO) {
                $editUrl = Route::_('index.php?option=com_jchoptimize&view=ModeSwitcher&task=setProduction&return='.\base64_encode(Uri::getInstance()), \false);
            } else {
                $editUrl = Route::_('index.php?option=com_plugins&filter[search]=JCH Optimize&filter[folder]=system');
            }
            $this->getApplication()->enqueueMessage(Text::sprintf('COM_JCHOPTIMIZE_PLUGIN_NOT_ENABLED', $editUrl), 'warning');
        }
        echo $this->view->render();
    }

    private function manageUpdates()
    {
        $this->updatesModel->upgradeLicenseKey();
        $this->updatesModel->refreshUpdateSite();
        $this->updatesModel->removeObsoleteUpdateSites();
        if (\JCH_PRO) {
            if ('' == $this->updatesModel->getLicenseKey()) {
                if (\version_compare(JVERSION, '4.0', 'lt')) {
                    $dlidEditUrl = Route::_('index.php?option=com_config&view=component&component=com_jchoptimize');
                } else {
                    $dlidEditUrl = Route::_('index.php?option=com_installer&view=updatesites&filter[search]=JCH Optimize&filter[supported]=1');
                }
                $this->getApplication()->enqueueMessage(Text::sprintf('COM_JCHOPTIMIZE_DOWNLOADID_MISSING', $dlidEditUrl), 'warning');
            }
        }
    }
}
